// iisadmin.cpp (Copyright 1998 by Niclas Schopenhauer)

#include "iisadmin.h"
#include "tchar.h"

// ADSI stuff
#include <activeds.h>

// --------------------------------------------------------------------------------------------------------
// 
// Use the following #import statement first. If it causes the following bug;
//   'activeds.tlh(243) : error C2079: '__MIDL_0015' uses undefined union '__MIDL___MIDL__intf_0000_0004'
// then modify the order of the definitions of __MIDL_0015 and __MIDL___MIDL__intf_0000_0004 in the
// generated activeds.tlh file. Then #include that file instead of the #import. 
// It's a MIDL bug in VC++ 5.
//
#include "Debug\activeds.tlh"	// Your path may be different
//#import "activeds.tlb"
// --------------------------------------------------------------------------------------------------------

// Filter definitions
#define METABASE_FILTER_ROOT					_T( "IIS://Localhost/W3svc/Filters" )
#define METABASE_FILTER_CLASS					_T( "IIsFilter" )
#define METABASE_FILTER_PATH					_T( "FilterPath" )
#define METABASE_FILTER_LOAD_ORDER				_T( "FilterLoadOrder" )

// Virtual directory definitions
#define METABASE_VIRTUAL_DIR_ROOT				_T( "IIS://Localhost/W3svc/1/ROOT" )
#define METABASE_VIRTUAL_ROOT_CLASS				_T( "IIsWebVirtualDir" )
#define METABASE_ALLOW_OUT_OF_PROC_COMPONENTS	_T( "AspAllowOutOfProcComponents" )
#define METABASE_PATH							_T( "Path" )
#define METABASE_TRUE							_T( "true" )
#define METABASE_FALSE							_T( "false" )


// Forward declarations
long InstallWebsite2( LPSTR virtDirPath, LPSTR physDirPath );
long RemoveWebsite2( LPSTR virtDirPath );
long InstallFilter2( LPSTR stringValue );




////////////////////////////////////////////////////////////////////////////////
//
// InstallFilter & InstallFilter2
//
// Purpose:		Installs an ISAPI filter by entering it in the metabase.
//
// Parameters:	filterPath		The path to the filter dll.
//
// Returns:		0				The filter was installed.
//
//				<0				The filter was not installed.
//
__declspec(dllexport) long InstallFilter( LPSTR filterPath )
{
	try
	{
		HRESULT		hr;
		long		result;

		// Initialize COM
		hr = CoInitialize(NULL);
		if ( FAILED( hr ) )
		{
			return hr;
		}

		// Need a function call since CoInitialize can't be done in the same scope
		// as the smartpointers are declared in
		result = InstallFilter2( filterPath );

		// Uninitialize
		CoUninitialize();

		return result;
	}
	catch( _com_error e )
	{
		return -99;
	}
	catch( ... ) 
	{
		return -98;
	}
}

long InstallFilter2( LPSTR filterPath )
{
	try
	{
		HRESULT						hr;				// Generic COM result variable
		_variant_t					variant;		// Variant used to set some properties in the metabase
		char*						dllName;		// The name of the filter dll to install
		ActiveDs::IADsContainerPtr	filterRoot;		// Filter root
		ActiveDs::IADsPtr			filterRootADs;	// Filter root
		ActiveDs::IADsPtr			newFilter;		// The new filter
		IDispatchPtr				dispatch;		// Temp interface pointer


		//
		// Get the filter root
		//
		hr = ADsGetObject(	METABASE_FILTER_ROOT, 
							IID_IADsContainer,
							( void** ) &filterRoot );
		if ( FAILED( hr ) )
		{
			return -2;
		}

		// Get the name of the dll (everything after the last '\')
		dllName = strrchr( filterPath, '\\' );
		if ( dllName == NULL )
		{
			// No '\' in the string, use the stringValue and see what happens
			dllName = filterPath;
		}
		else
		{
			// Get a pointer to the string after the '\', should be the dll name
			dllName++;
		}


		//
		// Create the new filter
		//
		dispatch = filterRoot->Create(	METABASE_FILTER_CLASS,	// Name of the filter schema class
										dllName	);				// Relative name of the object to create

		// Get the IADsContainer interface
		newFilter = dispatch;
		
		// Set the filter path switch
		variant = filterPath;
		newFilter->Put( METABASE_FILTER_PATH, variant );
		
		
		//
		// Set the filter load order
		//

		// Get the IADs interface to the filter root
		filterRootADs = filterRoot;

		// Get the load order
		variant = filterRootADs->Get( METABASE_FILTER_LOAD_ORDER );

		// Check if the filter dll is already in the load order
		if ( wcsstr( variant.bstrVal, _bstr_t( dllName ) ) == NULL )
		{
			variant.bstrVal = variant.bstrVal + _bstr_t( "," ) + dllName;
		}

		// Set the load order
		filterRootADs->Put( METABASE_FILTER_LOAD_ORDER, variant );

		//
		// Set the new values
		//
		newFilter->SetInfo();
		filterRootADs->SetInfo();

		return 0;
	}
	catch( _com_error e )
	{
		return -99;
	}
	catch( ... ) 
	{
		return -98;
	}
}


////////////////////////////////////////////////////////////////////////////////
//
// InstallWebsite & InstallWebsite2
//
// Purpose:		Installs a web site in IIS by entering it in the metabase.
//
// Parameters:	virtDirPath		a string containing the virtual directory.
//
//				physDirPath		a string containing the physical directory.
//
// Returns:		0				The web site was installed.
//
//				<0				The web site was not installed.
//
__declspec(dllexport) long InstallWebsite( LPSTR virtDirPath, LPSTR physDirPath )
{
	try
	{
		HRESULT		hr;
		long		result;

		// Initialize COM
		hr = CoInitialize(NULL);
		if ( FAILED( hr ) )
		{
			return hr;
		}

		// Need a function call since CoInitialize can't be done in the same scope
		// as the smartpointers are declared in
		result = InstallWebsite2( virtDirPath, physDirPath );

		// Uninitialize
		CoUninitialize();

		return result;
	}
	catch( _com_error e )
	{
		return -99;
	}
	catch( ... ) 
	{
		return -98;
	}
}

long InstallWebsite2( LPSTR virtDirPath, LPSTR physDirPath )
{
	try
	{
		HRESULT						hr;					// Generic COM result variable
		_variant_t					variant;			// Variant used to set some properties in the metabase
		ActiveDs::IADsContainerPtr	virtDirRoot;		// Virtual root
		ActiveDs::IADsPtr			virtDir;			// The new virtual directory
		IDispatchPtr				dispatch;			// Temp interface pointer


		//
		// Get the virtual directory root
		//
		hr = ADsGetObject(	METABASE_VIRTUAL_DIR_ROOT, 
							IID_IADsContainer,
							( void** ) &virtDirRoot );
		if ( FAILED( hr ) )
		{
			return hr;
		}

		//
		// Create the new virtual directory
		//
		dispatch = virtDirRoot->Create(	METABASE_VIRTUAL_ROOT_CLASS,	// Name of the virtual directory schema class
										virtDirPath );					// Relative name of the object to create

		// Get the IADsContainer interface
		virtDir = dispatch;

		// Set the AspAllowOutOfProcess switch
		variant = METABASE_TRUE;
		virtDir->Put( METABASE_ALLOW_OUT_OF_PROC_COMPONENTS, variant );
		
		// Set the physical path
		variant = physDirPath;
		virtDir->Put( METABASE_PATH, variant );
		
		//
		// Set the info
		//
		virtDir->SetInfo();
		
		return 0;
	}
	catch( _com_error e )
	{
		return -99;
	}
	catch( ... ) 
	{
		return -98;
	}
}


////////////////////////////////////////////////////////////////////////////////
//
// RemoveWebsite & RemoveWebsite2
//
// Purpose:		Removes a web site in IIS by removing it from the metabase.
//
// Parameters:	virtDirPath		a string containing the virtual directory.
//
// Returns:		0				The web site was removed.
//
//				<0				The web site was not removed.
//
__declspec(dllexport) long RemoveWebsite( LPSTR virtDirPath )
{
	try
	{
		HRESULT		hr;
		long		result;

		// Initialize COM
		hr = CoInitialize(NULL);
		if ( FAILED( hr ) )
		{
			return hr;
		}

		// Need a function call since CoInitialize can't be done in the same scope
		// as the smartpointers are declared in
		result = RemoveWebsite2( virtDirPath );

		// Uninitialize
		CoUninitialize();

		return result;
	}
	catch( _com_error e )
	{
		return -99;
	}
	catch( ... ) 
	{
		return -98;
	}
}

long RemoveWebsite2( LPSTR virtDirPath )
{
	try
	{
		HRESULT						hr;					// Generic COM result variable
		_variant_t					variant;			// Variant used to set some properties in the metabase
		ActiveDs::IADsContainerPtr	virtDirRoot;		// Virtual root
		ActiveDs::IADsPtr			virtDir;			// The new virtual directory
		IDispatchPtr				dispatch;			// Temp interface pointer


		//
		// Get the virtual directory root
		//
		hr = ADsGetObject(	METABASE_VIRTUAL_DIR_ROOT, 
							IID_IADsContainer,
							( void** ) &virtDirRoot );
		if ( FAILED( hr ) )
		{
			return hr;
		}

		//
		// Delete the virtual directory
		//
		hr = virtDirRoot->Delete(	METABASE_VIRTUAL_ROOT_CLASS,	// Name of the virtual directory schema class
									virtDirPath );					// Relative name of the object to remove

		if ( FAILED( hr ) )
		{
			return hr;
		}

#if 0
		// Get the IADsContainer interface
		virtDir = dispatch;

		// Set the AspAllowOutOfProcess switch
		variant = METABASE_TRUE;
		virtDir->Put( METABASE_ALLOW_OUT_OF_PROC_COMPONENTS, variant );
		
		// Set the physical path
		variant = physDirPath;
		virtDir->Put( METABASE_PATH, variant );
		
		//
		// Set the info
		//
		virtDir->SetInfo();
#endif	
		return 0;
	}
	catch( _com_error e )
	{
		return -99;
	}
	catch( ... ) 
	{
		return -98;
	}
}


